;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PROGRAM TESTOWY UKADU uC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; mruga na przemian diodami, na przewaniu odbiera i zwraca znak z RS232C/485
;; w przypadku gdy znak to spacja zmienia stan przekaznika

;; STAE

LED_GREEN	bit P1.2
LED_RED		bit P1.3
RELAY		bit P1.4

;; SKOK DO PROGRAMU GWNEGO

		jmp START



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PODPROGRAMY OBSUGI PRZERWA
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; SINT
;; obsuga interfejsu szeregowego

	org SINT

	clr TI
	jnb RI, sint_end ; obsluga tylko dla znaku przychodzacego

	push ACC
	mov A, SBUF	; odebranie znaku
	mov SBUF, A	; odeslanie znaku
	clr RI

	cjne A, #' ', sint_ok ; gdy spacja - zmien stan przekaznika
	cpl RELAY
sint_ok:
	pop ACC
sint_end:
	reti



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PROGRAM GOWNY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	org 100h

START:	
	call INIT

loop:	setb LED_GREEN
	clr LED_RED
	call PAUSE_1S
	clr LED_GREEN
	setb LED_RED
	call PAUSE_1S
	jmp loop



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PODPROGRAMY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; INIT
;; ogolna inicjacja ukadu
;; WE: brak
;; WY: brak

INIT:
	clr RELAY	; wyczenie przekanika
	clr LED_RED	; zgaszenie did
	clr LED_GREEN
	
	call RS_INIT	; inicjacja interfejsu szeregowego

	setb EA		; zaczenie obsugi przerwa

	ret



;; PAUSE_1S 
;; programowa przerwa o dugoci 1 sek. 
;; WE: brak
;; WY: brak

PAUSE_1S:
	push 0
	push 1

	mov R0, #250	; wartosci skalkulowane dla kwarca 6MHz
p1s_l1: mov R1, #250
p1s_l2: nop		
	nop
	nop
	nop
	nop
	nop
        djnz R1, p1s_l2	; ((6 * nop + 1 * djnx)=8 taktow) * 250 = 2000 taktow
	djnz R0, p1s_l1	; (2000 taktow * 250 = 500000 taktow) * (2 us na takt) = 1 s

	pop 1
	pop 0
	ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PODPROGRAMY OBSUGI INTERFEJSU SZEREGOWEGO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; RS_INIT
;; inicjuje interfejs szeregowy - tryb 1, prdko transmisji 1200bps
;; WE: brak
;; WY: brak

RS_INIT:
	push ACC

	clr SM0		; tryb 1
	setb SM1
	setb REN	; zacz odbiornik
	setb ES		; zgaszanie przerwania z interfejsu

	mov TH1, #0f3h	; warto zegara taktujcego dla 1200bps i 6MHz
	mov TL1, #0f3h	
	mov A, TMOD
	anl A, #0fh
	orl A, #20h
	mov TMOD, A	; zegar T1 w trybie 2
	setb TR1

	pop ACC
	ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; KONIEC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	end
